;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xf9717f416d68396c364b90aa847ccf0b . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x82f4cad56c7904bb8de610d3a1d85c1a . "./code.cg2js/cg-gesture-emulation.fasl")
(#xc14768ba0f3954cfa8c5cd029c666e4e . "./code.cg2js/cg-grid-widget.fasl")
(#x75820ee986b3629b05e6888ce65fae46 . "./code.cg2js/cg-utility-dialog.fasl")
(#x0135d9eabb66059e5ccd537b67d69830 . "./code.cg2js/cg-wrap-string.fasl")
(#x96dac8400312dc412b086be41eb86505 . "./code.cg2js/cg-editable-text.fasl")
(#x3a198996654a8fc1edf62fb568acda88 . "./code.cg2js/cg-base.fasl")
(#x658e179b19fa9b4874f74f17486c5507 . "./code.cg2js/cg-lisp-text.fasl")
(#x966740de53ca2de451f248f351c3621b . "./code.cg2js/cg-intersect.fasl")
(#x3fa053a8df1252e973344b9821d3d346 . "./code.cg2js/cg-choose-printer.fasl")
(#xf1a9b3b07d87910bcceeca42060d01ae . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x21678c0339b40001331282d5f99bc3e0 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x1951956b6241b2f550de0af6b06b08c8 . "./code.cg2js/cg-font-dialog.fasl")
(#x1fd653967a266b28ae32818ede828533 . "./code.cg2js/cg-comtab.fasl")
(#xfb71221caa679e07804a955876bbcb78 . "./code.cg2js/cg-tooltip.fasl")
(#xe396b755123d7ed542dc79a8b6fb2baf . "./code.cg2js/cg-chart-or-plot.fasl")
(#x673a58729f09e499178f1f7f56047d8e . "./code.cg2js/cg-get-pixmap.fasl")
(#x72a0c14149bbc8c3793b77b0258cb60b . "./code.cg2js/cg-status-bar.fasl")
(#xc3af875d3d33d230313594798cce9b0b . "./code.cg2js/cg-tab-control.fasl")
(#x34c051a7389ccc4158909b81438970bc . "./code.cg2js/cg-icon-pixmap.fasl")
(#xb5ef8dc1a4696c0f81df9d37f0de84f2 . "./code.cg2js/cg-list-view.fasl")
(#xa859a725596273f11c84a2f66177df7d . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x92339c4b73f7853ecf91bb374ae2d3a6 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xad470902bfe91f6876a706bfe79f3e0c . "./code.cg2js/cg-project-window.fasl")
(#x0bcff83265df6833218f581f0a553c3b . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x7ecf5531b8b7e687db56c7010f47bc98 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x7b6b0b0f054925bbbb0efa9e4745a7ae . "./code.cg2js/cg-scaling-stream.fasl")
(#x7dae8e0411f768e493d947c9e7086327 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x02728d437ee390f153b05adfceeb456d . "./code.cg2js/cg-menu.fasl")
(#x079adb019efbc909254b81b940f5ebf8 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x46d9323659e67dc0080ee60fe3098f13 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x6492a55db66e025688d91e2bc3272020 . "./code.cg2js/cg-split-bar.fasl")
(#x5f554bfe4e1f6619af388b1547ac8cf7 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x0e0bbac37775fddab91feec5930cc42e . "./code.cg2js/cg-timer.fasl")
(#x839dbe78a43107e41e54449fe17f2630 . "./code.cg2js/cg-text-or-combo.fasl")
(#x2f4ee95b8e1106275d568534fa6a65ca . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x301c0014084d76e08a6d8bde4b3f35a5 . "./code.cg2js/cg-string-dialog.fasl")
(#xc152a4fa36ce98eaf4537aa412a441b1 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x1b41ea39b3999d636931dee4509c8eeb . "./code.cg2js/cg-menu-tooltip.fasl")
(#xa354d0554ff743cabe0326f55847e0af . "./code.cg2js/cg-outline.fasl")
(#xeccc270ff9a5751a491aa79fccfc9a45 . "./code.cg2js/cg-lettered-menu.fasl")
(#x171259e8a627b102e3d903f34e256416 . "./code.cg2js/cg-graphics-context.fasl")
(#x575eadb6fc91dc45c83e92c22ee10ed7 . "./code.cg2js/cg-item-list.fasl")
(#xfe100163ecf523271435e5177fcfefb4 . "./code.cg2js/cg-caret.fasl")
(#x48f2268ddadb0824fa1df4e5cd1d7208 . "./code.cg2js/cg-printing.fasl")
(#xd2e36c8ca752aba4d4b75247540ad28a . "./code.cg2js/cg-pixmap-printing.fasl")
(#x0067858ffff0fb57dcd1dea0781fd76a . "./code.cg2js/cg-progress-indicator.fasl")
(#xaca36e5a7746565b01d225865d2bce0f . "./code.cg2js/cg-dde-utils.fasl")
(#xda0979789fa862ad7e5b1b747794d1e0 . "./code.cg2js/cg-plot-widget.fasl")
(#x0725a507c9db73e8497f98c946f0fb47 . "./code.cg2js/cg-os-window.fasl")
(#x5328c66008074e4d9bda120ffeb40c14 . "./code.cg2js/cg-calendar.fasl")
(#xab5a8d2e671d5ae646c69fbde1f4530d . "./code.cg2js/cg-chart-widget.fasl")
(#x74b456fc33cbc9619c0050a73c86c3c5 . "./code.cg2js/cg-file-dialog.fasl")
(#x846669ed3728bda726de0db14cc8af60 . "./code.cg2js/cg-nodes.fasl")
(#x5911f0f9a49d8d374b2911ef835ecbc2 . "./code.cg2js/cg-lamp.fasl")
(#xb094832f7f0e69ebb1b0298f7a1ff108 . "./code.cg2js/cg-html-dialog.fasl")
(#xf97a88b8b73f1d2f71e8f30ebaaaae5f . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x778016abe65d098a27e0470f0f1cefce . "./code.cg2js/cg-class-slot-grid.fasl")
(#x15a4d500e11f6e6d4691ea2c0ed0aa8a . "./code.cg2js/cg-choice-list.fasl")
(#x4633affa14df1bb2217e0f2c39505a16 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x735f585c3fe1915f3515d81886641ef7 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x92a7621b3f1c7b6c6ba745b16c5affe8 . "./code.cg2js/cg-hotspot.fasl")
(#x25c614fada54479569038282ff65c910 . "./code.cg2js/cg-property.fasl")
(#x243f8f3d0d550ad074edcda79241724a . "./code.cg2js/cg-dropping-outline.fasl")
(#x1e5e420aef88f08b545cd9e94ebcfa82 . "./code.cg2js/cg-template-string.fasl")
(#x953bddff6f43d2918ed3433add0b01eb . "./code.cg2js/cg-palette.fasl")
(#x746f387d428539d0fd46cef73ccabe76 . "./code.cg2js/cg-selected-object.fasl")
(#xcff07de5e04eb1b069c93a7cb0d26839 . "./code.cg2js/cg-up-down-control.fasl")
(#x2ea0c9e3e91e542f45c8fc9a459ba32d . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x08b487612c56703dc2e320715f3e9e77 . "./code.cg2js/cg-layout.fasl")
(#x6bca80d5c293508374966446eb489a73 . "./code.cg2js/cg-get-position.fasl")
(#x974d230ab430bad9c8168d81f166ec54 . "./code.cg2js/cg-lisp-widget.fasl")
(#x23267da8991bacbee61aa57d08faa6d4 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x3916cf387774c2ecf776249cb0e1c55f . "./code.cg2js/cg-object-editor.fasl")
(#x4344121805fc06290ad750df095cdb36 . "./code.cg2js/cg-class-grid.fasl")
(#xe2d9fffa37ffca10eb8adfe025d4b0d8 . "./code.cg2js/cg-yes-no-string.fasl")
(#x938d29707e104bdab8427cc7eec6fbcd . "./code.cg2js/cg-toggling-widget.fasl")
(#x4803c78ad8adb7eb4e602d86fd25c1e3 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x937a1efbb63b0afdd61fe558c4907e8c . "./code.cg2js/cg-button.fasl")
(#x9ad8f754b764371cd07229a222618ddc . "./code.cg2js/cg-object-editor-layout.fasl")
(#x0f8ea6e42e8e258c2fb45ae7e4ee8320 . "./code.cg2js/cg-scroll-bar.fasl")
(#xf400ec479e4c22d937cce393523d2022 . "./code.cg2js/cg-combo-box.fasl")
(#x5360d07738ae6a2d3103d828f39ad061 . "./code.cg2js/cg-web-browser.fasl")
(#xc6274a07239d83ae7c2f1e126351d6c6 . "./code.cg2js/cg-drawable.fasl")
(#x8b8e01daf400964f5342ed7b03ca2628 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xb3bd14bccf65e5cf9b362823dd932ccc . "./code.cg2js/cg-message-dialog.fasl")
(#xee274a1ae8d2160c767243234a2aea75 . "./code.cg2js/cg-text-widget.fasl")
(#x2eddae0609b8d0260ab0c515e7a90dfc . "./code.cg2js/cg-static-text.fasl")
(#x9cca3fcdd7d4d329aaa61663790e8004 . "./code.cg2js/cg-class-support.fasl")
(#x7fe440755da10a1c9b7947dd420263b8 . "./code.cg2js/cg.fasl")
(#x609a5c1b2d1f191aed096ec98e537ed9 . "./code.cg2js/cg-directory-dialog.fasl")
(#x5ecd58f46d9a688e6426639a5b85fa15 . "./code.cg2js/cg-clipboard.fasl")
(#xd082d4bdea5c0929674fbd41ef2b825a . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x831b57b1afca98772d1ea00d7e99ce6e . "./code.cg2js/cg-bitmap-pane.fasl")
(#xf6f8537f9ca22da5715172b3aedab127 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x092ff0f5992517ccba51de7469272254 . "./code.cg2js/cg-check-box.fasl")
(#xa77a4630ba918817b45ee612ecea868f . "./code.cg2js/cg-color-dialog.fasl")
(#x0b2c7a2f6640e33a7e4b678231024d0c . "./code.cg2js/cg-text-edit-pane.fasl")
(#x255244c5cf52d5cdff3d0e3bb957a57a . "./code.cg2js/cg-dialog-item.fasl")
(#x904775d65c5e3efc99c2aeb2409ecf17 . "./code.cg2js/cg-edit-in-place.fasl")
(#xacafc18a1a311590785f168f5a8320f3 . "./code.cg2js/cg-tray.fasl")
(#x2997dbdcc4a7f8a9821473865174f048 . "./code.cg2js/cg-paren-matching.fasl")
(#x0d3b8feacbe52bae1b856f1cfdf88d19 . "./code.cg2js/cg-player.fasl")
(#x791055517f82f660054ad2a4ae769d2d . "./code.cg2js/cg-yes-no-list.fasl")
(#xec08ecdf12dde1d011b75c927ca233c7 . "./code.cg2js/cg-common-control.fasl")
(#xb712f4b000152ddeaf64a8c7b2e08da4 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x82ca8c7af87b320c25c7a926473adb2a . "./code.cg2js/cg-os-widget.fasl")
(#x82cd3e8d897de1088a42699c9db6c023 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x933b741252f57e34d7f96f38c4c6b5bb . "./code.cg2js/cg-trackbar.fasl")
(#xf34e3992e8238a376b679b488826082d . "./code.cg2js/cg-find-string-dialog.fasl")
(#xdbeb8ecd814e689e8dbdddf7f375a210 . "./code.cg2js/cg-html-widget.fasl")
(#xb80a7e789786ab5ade094b67055b551d . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x0d4fe3372d55c234326eda106c75c827 . "./code.cg2js/cg-acache.fasl")
(#xd7167205bab6b13ffcd08785e8ef7c63 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xb0cad61258a5e7f5efb52df5f4dba150 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x669258af60a6ace3a9cda82b29d23d0a . "./code.cg2js/cg-icon.fasl")
(#x8177e6e7cc11ec2d2dbe4a5f26c308fd . "./code.cg2js/cg-pixmap.fasl")
(#x1db1815c76f11bf2fdd5ab569d122c88 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x73d5368668c2dab011cae18605150d18 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x3a369c8a5bd10408ce9f694c23f9746f . "./code.cg2js/cg-toolbar.fasl")
(#x03443791ea90177319f8f4dd8ebf9e50 . "./code.cg2js/cg-radio-button.fasl")
(#x498aa731f7cc0ad554d418ad35422f2c . "./code.cg2js/cg-curve.fasl")
(#xa3ad042c92a172c93a9f016c31938e76 . "./code.cg2js/cg-picture-widget.fasl")
(#x05e32f2c08d558c971c641de6d3e87ff . "./code.cg2js/cg-dev.fasl")
(#xe6d4a3238c548b6878752ade6a3c99aa . "./code.cg2js/cg-group-box.fasl")
(#x41200a7415dd2c1aedbf1fb8b6fb6477 . "./code.cg2js/cg-fill-texture.fasl")
(#x3e43004d90f8d1d9380a6c59548cbb5d . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x3d11f0666debf83625bdb3c69b9f5bbe . "./code.idejs/ide-keyshort.fasl")
(#x5077ba700138b378f5a49e90bd948161 . "./code.idejs/ide-form.fasl")
(#xb3a2358f2afa3f7313e68152b4488df3 . "./code.idejs/ide-find-in-files.fasl")
(#x9c28d8b837774ee20ca07157b586ac9a . "./code.idejs/ide-class-grapher.fasl")
(#x9c81acbed765014464efde944262d211 . "./code.idejs/ide-base.fasl")
(#x98b9bd42174cd7b8fe61a7d0dcd14a8b . "./code.idejs/ide-class-browser.fasl")
(#x3964460eec5857d64f1e1566011ba2d5 . "./code.idejs/ide-find-definitions.fasl")
(#xe77d6d5271a426ee6082d191cb09775e . "./code.idejs/ide-package-list.fasl")
(#x129094a7a496b7c1e5e98e18615826a1 . "./code.idejs/ide-grapher.fasl")
(#x1a644d1bb1cc4b1013bb76687ef66538 . "./code.idejs/ide-dev.fasl")
(#x2bb196fb98e72275737514c7128b6ff2 . "./code.idejs/ide.fasl")
(#x8821d5efcdada73caae0b3adc28b3019 . "./code.idejs/ide-trace.fasl")
(#x1ff13f97a00972b49bb6d7156fafb143 . "./code.idejs/ide-apropos.fasl")
(#x62fa1e72da06f5e46733b4d3d1328693 . "./code.idejs/ide-inspector.fasl")
(#xd07af9ee564cdd712861011df48ed923 . "./code.idejs/ide-acache.fasl")
(#x326206dae16592111013285b5f928cb7 . "./code.idejs/ide-project.fasl")
(#xb23c8fc9e2b0db4cf74aaa7a3a061c57 . "./code.idejs/ide-window-list.fasl")
(#xbef12a25ff479fb2c31a6ef7c1079261 . "./code.idejs/ide-intro.fasl")
(#x576f47eb16a3e8c10ab94a16cafc5c53 . "./code.idejs/ide-clipboard.fasl")
(#xbba4149eddf4caea4c0981152ff31368 . "./code.idejs/ide-editor.fasl")
(#x355249cd2d6b47a286585e1f36dbc1f5 . "./code.idejs/ide-process-browser.fasl")
(#xc6890680581284b530438ff9ff69c756 . "./code.idejs/ide-patch-dialog.fasl")
(#xc89691f4dab1c1d81ba932af073979a9 . "./code.idejs/ide-navigator.fasl")
(#xf3f59eb0bf604c0b34476c9897472f9c . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
